/*
 * (C) Copyright 2001
 * Denis Peter, MPL AG Switzerland, d.peter@mpl.ch
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 * hacked for PIP405
 */

#include <ppcboot.h>
#include <command.h> 
#include "mip405.h"
#include <rtc.h>
#include "../common/isa.h"
#include "../common/video.h"
#include <i2c.h>
 
extern flash_info_t flash_info[];	/* info for FLASH chips */

int PCI_Write_CFG_Reg(int BusDevFunc, int Reg, unsigned int Value, int Width);
unsigned int PCI_Read_CFG_Reg(int BusDevFunc, int Reg, int Width);
extern void print_mip405_info(void);





#define TRUE 1
#define FALSE 0

#define IMAGE_SIZE 0x80000 

image_header_t header;



int mip405_prg(unsigned long src,unsigned long size)
{
	unsigned long start;
	flash_info_t *info;
	int i,rc;
	unsigned long *magic = (unsigned long *)src;
	
	info = &flash_info[0];
  	start = 0 - size;
	for(i=info->sector_count-1;i>0;i--)
	{
		info->protect[i] = 0; /* unprotect this sector */
		if(start>=info->start[i])
		break;
	}
	/* set-up flash location */
	/* now erase flash */
	if(magic[0]!=IH_MAGIC) {
		printf("Bad Magic number\n");
		return -1;
	}
	printf("Start erasing at %lx (sector %d) (start %lx)\n",
				start,i,info->start[i]);
	flash_erase (info, i, info->sector_count-1);
	printf("flash erased, programming from 0x%lx 0x%lx Bytes\n",src,size);
	switch (rc = flash_write ((uchar *)src, start, size)) 
	{
		case 0:	printf ("programming done\n"); return 0;
		case 1: printf ("Timeout writing to Flash\n"); return 1;
		case 2:	printf ("Flash not Erased\n"); return 1;
		case 4: printf ("Can't write to protected Flash sectors\n"); return 1;
		case 8: printf ("Outside available Flash\n"); return 1;
		default:	printf ("%s[%d] FIXME: rc=%d\n",__FILE__,__LINE__,rc); return 1;
	}
  return 1;
}


int mip405_prg_image(unsigned long load_addr)
{
	unsigned long data,len,checksum;
	image_header_t *hdr=&header;  
	/* Copy header so we can blank CRC field for re-calculation */
	memcpy (&header, (char *)load_addr, sizeof(image_header_t));

	if (hdr->ih_magic  != IH_MAGIC) {
		printf ("Bad Magic Number\n");
		return 1;
	}
	if (hdr->ih_os  != IH_OS_PPCBOOT) {
		printf ("No PPCBOOT Image\n");
		return 1;
	}
	if (hdr->ih_type  != IH_TYPE_FIRMWARE) {
		printf ("No Firmware Image\n");
		return 1;
	}
	data = (ulong)&header;
	len  = sizeof(image_header_t);
	checksum = hdr->ih_hcrc;
	hdr->ih_hcrc = 0;
	if (crc32 (0, (char *)data, len) != checksum) {
		printf ("Bad Header Checksum\n");
		return 1;
	}
	data = load_addr + sizeof(image_header_t);
	len  = hdr->ih_size;
	printf ("   Verifying Checksum ... ");
	if (crc32 (0, (char *)data, len) != hdr->ih_dcrc) {
		printf ("Bad Data CRC\n");
		return 1;
	}
	printf ("OK\n");
	return(mip405_prg(data,len));
} 


/* ------------------------------------------------------------------------- */

int do_mip405(cmd_tbl_t *cmdtp, bd_t *bd, int flag, int argc, char *argv[])
{

 	ulong size,src,device,load_addr;
	int result;	
	backup_t back;
	src = MULTI_PURPOSE_SOCKET_ADDR;
	size = IMAGE_SIZE;

	if (strcmp(argv[1], "flash") == 0) 
	{
		if (strcmp(argv[2], "mem") == 0) {
	    		if(argc==4) {
				load_addr=simple_strtoul(argv[3], NULL, 16);
			}
			else {
				load_addr=CFG_LOAD_ADDR;
			}
			printf ("\nupdating bootloader image from memory at %lX\n",load_addr);
			result=mip405_prg_image(load_addr); 
			return result;
		}
		if (strcmp(argv[2], "mps") == 0) {
			printf ("\nupdating bootloader image from MSP\n");
			result=mip405_prg(src,size);
			return result;
		}
		
	}
	if (strcmp(argv[1], "mem") == 0) 
	{
		result=0;
		if(argc==3)
		{
			result = (int)simple_strtol(argv[2], NULL, 16);
	    }
	    src=(unsigned long)&result;
	    src-=CFG_MEMTEST_START;
	    src-=(100*1024); /* - 100k */
	    src&=0xfff00000;
	    size=0;
	    do {
	    	size++;
			printf("Pass %ld\n",size);
			mem_test(CFG_MEMTEST_START,src,1);
			if(ctrlc())
				break;
			if(result>0)
				result--;
			
		}while(result);
		return 0;
	}
 	if (strcmp(argv[1], "info") == 0) 
	{
		print_mip405_info();
	 	return 0;
   	}
 	if (strcmp(argv[1], "led") == 0) 
	{
		device = (ulong)simple_strtoul(argv[2], NULL, 10);
		user_led0((device==1));
		return 0;
   	}
	if (strcmp(argv[1], "getback") == 0) {
		get_backup_values(&back);
		back.signature[3]=0;
		back.serial_name[16]=0;
		back.eth_addr[20]=0;
		printf("GetBackUp: signature: %s\n",back.signature);
		printf("           serial#:   %s\n",back.serial_name);
		printf("           ethaddr:   %s\n",back.eth_addr);
		return 0;
	} 
	if (strcmp(argv[1], "setback") == 0) {
		set_backup_values(1);
		return 0;
	} 
	printf("Usage:\n%s\n", cmdtp->usage);
    return 1;
}

/* ------------------------------------------------------------------------- */
